package data.missions.xlu_burningsensation;

import com.fs.starfarer.api.fleet.FleetGoal;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.mission.FleetSide;
import com.fs.starfarer.api.mission.MissionDefinitionAPI;
import com.fs.starfarer.api.mission.MissionDefinitionPlugin;

public class MissionDefinition implements MissionDefinitionPlugin {

	public void defineMission(MissionDefinitionAPI api) {

		// Set up the fleets
		api.initFleet(FleetSide.PLAYER, "XLU", FleetGoal.ATTACK, false, 10);
		api.initFleet(FleetSide.ENEMY, "HSS", FleetGoal.ATTACK, true, 10);

		// Set a blurb for each fleet
		api.setFleetTagline(FleetSide.PLAYER, "XLU Marekelig-family assault convoy");
		api.setFleetTagline(FleetSide.ENEMY, "Hegemony patrol fleet, under Onslaught captain Argus Lycan");
		
		// These show up as items in the bulleted list under 
		// "Tactical Objectives" on the mission detail screen
		api.addBriefingItem("Defeat all enemy forces");
		api.addBriefingItem("XLU Standford IV must survive");
		api.addBriefingItem("Harry the Hegemony fast, so we can get out of this mess.");
		api.addBriefingItem("That Onslaught just took out John, get him first.");
		
		// Set up the player's fleet
		api.addToFleet(FleetSide.PLAYER, "xlu_cobalt_Assault", FleetMemberType.SHIP, "XLU Standford IV", true);
		api.addToFleet(FleetSide.PLAYER, "xlu_ferros_Barrage", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "xlu_stibium_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "xlu_lithium_Strike", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "xlu_lead_Shock", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "xlu_zinc_Shock", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.PLAYER, "xlu_alum_Strike", FleetMemberType.SHIP, false);
		
		// Mark player flagship as essential
		api.defeatOnShipLoss("XLU Standford IV");
		
		// Set up the enemy fleet
		api.addToFleet(FleetSide.ENEMY, "onslaught_Outdated", FleetMemberType.SHIP, "HSS Vermount II", true);
		api.addToFleet(FleetSide.ENEMY, "dominator_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "dominator_Support", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "enforcer_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "enforcer_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "enforcer_CS", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "lasher_CS", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "lasher_CS", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "lasher_CS", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "lasher_CS", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "lasher_Assault", FleetMemberType.SHIP, false);
		api.addToFleet(FleetSide.ENEMY, "lasher_Assault", FleetMemberType.SHIP, false);
		
		// Set up the map.
		float width = 14000f;
		float height = 12000f;
		api.initMap((float)-width/2f, (float)width/2f, (float)-height/2f, (float)height/2f);
		
		float minX = -width/2;
		float minY = -height/2;
		
		for (int i = 0; i < 15; i++) {
			float x = (float) Math.random() * width - width/2;
			float y = (float) Math.random() * height - height/2;
			float radius = 100f + (float) Math.random() * 900f; 
			api.addNebula(x, y, radius);
		}
		
		api.addNebula(minX + width * 0.8f - 1000, minY + height * 0.2f, 1000);
		api.addNebula(minX + width * 0.8f - 1000, minY + height * 0.3f, 1000);
		api.addNebula(minX + width * 0.8f - 1000, minY + height * 0.4f, 1000);
		
		api.addObjective(minX + width * 0.25f + 1000, minY + height * 0.25f + 2000, "nav_buoy");
		api.addObjective(minX + width * 0.75f - 1000, minY + height * 0.25f + 2000, "comm_relay");
		api.addObjective(minX + width * 0.7f, minY + height * 0.5f, "sensor_array");
		api.addObjective(minX + width * 0.3f + 1000, minY + height * 0.5f, "sensor_array");
		
		// Add an asteroid field
		api.addAsteroidField(minX + width * 0.3f, minY, 90, 3000f,
								20f, 70f, 50);
		
		// Add some planets.  These are defined in data/config/planets.json.
		api.addPlanet(0, 0, 200f, "lava", 350f, true);
	}

}






